@@ -1,7 +1,7 @@
 """Generate langauge specific loaders for a particular SALAD schema."""
 import sys
 from io import open
-from typing import Any, Dict, List, MutableMapping, Optional
+from typing import Any, Dict, List, MutableMapping, MutableSequence, Optional
 
 from . import schema
 from .codegen_base import CodeGenBase
@@ -14,13 +14,13 @@ from .utils import aslist
 
 
 def codegen(
-    lang,  # type: str
-    i,  # type: List[Dict[str, Any]]
-    schema_metadata,  # type: Dict[str, Any]
+    lang: str,
+    i: List[Dict[str, str]],
+    schema_metadata: Dict[str, Any],
     loader: Loader,
-    target=None,  # type: Optional[str]
-    examples=None,  # type: Optional[str]
-):  # type: (...) -> None
+    target: Optional[str] = None,
+    examples: Optional[str] = None,
+) -> None:
     """Generate classes with loaders for the given Schema Salad description."""
 
     j = schema.extend_and_specialize(i, loader)
@@ -63,8 +63,16 @@ def codegen(
                 document_roots.append(rec["name"])
 
             field_names = []
+            optional_fields = set()
             for field in rec.get("fields", []):
-                field_names.append(shortname(field["name"]))
+                field_name = shortname(field["name"])
+                field_names.append(field_name)
+                tp = field["type"]
+                if (
+                    isinstance(tp, MutableSequence)
+                    and tp[0] == "https://w3id.org/cwl/salad#null"
+                ):
+                    optional_fields.add(field_name)
 
             idfield = ""
             for field in rec.get("fields", []):
@@ -78,6 +86,7 @@ def codegen(
                 rec.get("abstract", False),
                 field_names,
                 idfield,
+                optional_fields,
             )
             gen.add_vocab(shortname(rec["name"]), rec["name"])
 
