@@ -21,7 +21,7 @@ from urllib.request import pathname2url
 from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap
 from schema_salad.exceptions import SchemaSaladException, ValidationException
-from schema_salad.ref_resolver import Fetcher
+from schema_salad.fetcher import DefaultFetcher, Fetcher
 from schema_salad.sourceline import SourceLine, add_lc_filename
 
 _vocab = {}  # type: Dict[str, str]
@@ -44,6 +44,7 @@ class LoadingOptions(object):
         self,
         fetcher=None,  # type: Optional[Fetcher]
         namespaces=None,  # type: Optional[Dict[str, str]]
+        schemas=None,  # type: Optional[Dict[str, str]]
         fileuri=None,  # type: Optional[str]
         copyfrom=None,  # type: Optional[LoadingOptions]
         original_doc=None,  # type: Optional[Any]
@@ -51,6 +52,7 @@ class LoadingOptions(object):
         self.idx = {}  # type: Dict[str, Dict[str, Any]]
         self.fileuri = fileuri  # type: Optional[str]
         self.namespaces = namespaces
+        self.schemas = schemas
         self.original_doc = original_doc
         if copyfrom is not None:
             self.idx = copyfrom.idx
@@ -60,12 +62,13 @@ class LoadingOptions(object):
                 self.fileuri = copyfrom.fileuri
             if namespaces is None:
                 self.namespaces = copyfrom.namespaces
+            if schemas is None:
+                self.schemas = copyfrom.schemas
 
         if fetcher is None:
             import requests
             from cachecontrol.wrapper import CacheControl
             from cachecontrol.caches import FileCache
-            from schema_salad.ref_resolver import DefaultFetcher
 
             if "HOME" in os.environ:
                 session = CacheControl(
@@ -148,16 +151,14 @@ def save(
 
 
 def expand_url(
-    url,  # type: Union[str, str]
-    base_url,  # type: Union[str, str]
+    url,  # type: str
+    base_url,  # type: str
     loadingOptions,  # type: LoadingOptions
     scoped_id=False,  # type: bool
     vocab_term=False,  # type: bool
     scoped_ref=None,  # type: Optional[int]
 ):
     # type: (...) -> str
-    url = str(url)
-
     if url in ("@id", "@type"):
         return url
 
@@ -382,8 +383,10 @@ class _TypeDSLLoader(_Loader):
         # type: (...) -> Union[List[Union[Dict[str, str], str]], Dict[str, str], str]
         m = self.typeDSLregex.match(doc)
         if m:
+            group1 = m.group(1)
+            assert group1 is not None
             first = expand_url(
-                m.group(1), baseuri, loadingOptions, False, True, self.refScope
+                group1, baseuri, loadingOptions, False, True, self.refScope
             )
             second = third = None
             if bool(m.group(2)):
@@ -467,11 +470,13 @@ def _document_load(loader, doc, baseuri, loadingOptions):
         )
 
     if isinstance(doc, MutableMapping):
-        if "$namespaces" in doc:
+        if "$namespaces" in doc or "$schemas" in doc:
             loadingOptions = LoadingOptions(
-                copyfrom=loadingOptions, namespaces=doc["$namespaces"]
+                copyfrom=loadingOptions,
+                namespaces=doc.get("$namespaces", None),
+                schemas=doc.get("$schemas", None),
             )
-            doc = {k: v for k, v in doc.items() if k != "$namespaces"}
+            doc = {k: v for k, v in doc.items() if k not in ["$namespaces", "$schemas"]}
 
         if "$base" in doc:
             baseuri = doc["$base"]
