@@ -4,6 +4,7 @@
 
 import difflib
 import re
+from typing import Any
 
 from schema_salad.utils import json_dumps
 
@@ -16,18 +17,18 @@ class JsonDiffMatcher(object):
     include non-trivial data structures.
     """
 
-    def __init__(self, expected):
+    def __init__(self, expected: Any):
         self.expected = expected
 
-    def __eq__(self, actual):
+    def __eq__(self, actual: Any) -> bool:
         expected_json = json_dumps(self.expected, sort_keys=True, indent=2)
         actual_json = json_dumps(actual, sort_keys=True, indent=2)
         if expected_json != actual_json:
             raise AssertionError(
                 "".join(
                     difflib.context_diff(
-                        expected_json.splitlines(1),
-                        actual_json.splitlines(1),
+                        expected_json.splitlines(True),
+                        actual_json.splitlines(True),
                         fromfile="Expected",
                         tofile="Actual",
                     )
@@ -36,5 +37,5 @@ class JsonDiffMatcher(object):
         return True
 
 
-def StripYAMLComments(yml):
+def StripYAMLComments(yml: str) -> Any:
     return re.sub(r"(?ms)^(#.*?\n)*\n*", "", yml)
