@@ -1,8 +1,38 @@
 import json
 import os
-from typing import IO, Any, Dict, Mapping, MutableSequence
+from typing import (
+    IO,
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Mapping,
+    MutableSequence,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
-# move to a regular typing import when Python 3.3-3.6 is no longer supported
+import requests
+from rdflib.graph import Graph
+
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
+
+if TYPE_CHECKING:
+    from .fetcher import Fetcher
+
+ContextType = Dict[str, Union[Dict[str, Any], str, Iterable[str]]]
+DocumentType = TypeVar("DocumentType", CommentedSeq, CommentedMap)
+DocumentOrStrType = TypeVar("DocumentOrStrType", CommentedSeq, CommentedMap, str)
+FieldType = TypeVar("FieldType", str, CommentedSeq, CommentedMap)
+ResolveType = Union[int, float, str, CommentedMap, CommentedSeq, None]
+ResolvedRefType = Tuple[ResolveType, CommentedMap]
+IdxResultType = Union[CommentedMap, CommentedSeq, str, None]
+IdxType = Dict[str, IdxResultType]
+CacheType = Dict[str, Union[str, Graph, bool]]
+FetcherCallableType = Callable[[CacheType, requests.sessions.Session], "Fetcher"]
+AttachmentsType = Callable[[Union[CommentedMap, CommentedSeq]], bool]
 
 
 def add_dictlist(di, key, val):  # type: (Dict[Any, Any], Any, Any) -> None
@@ -11,31 +41,31 @@ def add_dictlist(di, key, val):  # type: (Dict[Any, Any], Any, Any) -> None
     di[key].append(val)
 
 
-def aslist(l):  # type: (Any) -> MutableSequence[Any]
+def aslist(thing):  # type: (Any) -> MutableSequence[Any]
     """
     Convenience function to wrap single items and lists.
 
     Return lists unchanged.
     """
 
-    if isinstance(l, MutableSequence):
-        return l
+    if isinstance(thing, MutableSequence):
+        return thing
     else:
-        return [l]
+        return [thing]
 
 
 # http://rightfootin.blogspot.com/2006/09/more-on-python-flatten.html
 
 
-def flatten(l, ltypes=(list, tuple)):
+def flatten(thing, ltypes=(list, tuple)):
     # type: (Any, Any) -> Any
-    if l is None:
+    if thing is None:
         return []
-    if not isinstance(l, ltypes):
-        return [l]
+    if not isinstance(thing, ltypes):
+        return [thing]
 
-    ltype = type(l)
-    lst = list(l)
+    ltype = type(thing)
+    lst = list(thing)
     i = 0
     while i < len(lst):
         while isinstance(lst[i], ltypes):
